"""
django-braces mixins library
----------------------------

Mixins to make Django's generic class-based views simpler.

:copyright: (c) 2013 by Kenneth Love and Chris Jones
:license: BSD 3-clause. See LICENSE for more details
"""

__title__ = "braces"
__version__ = "1.17.0"
__author__ = "Kenneth Love and Chris Jones"
__license__ = "BSD 3-clause"
__copyright__ = "Copyright 2013 Kenneth Love and Chris Jones"
